// Vikunja is a to-do list application to facilitate your life.
// Copyright 2018-2020 Vikunja and contributors. All rights reserved.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <https://www.gnu.org/licenses/>.

package models

import (
	"time"

	uuid "github.com/satori/go.uuid"

	"code.vikunja.io/api/pkg/user"
)

// OAuth2Client represents a third party application used the authenticate with Vikunja
type OAuth2Client struct {
	ClientID      string `xorm:"varchar(36) pk not null"`
	ClientSecret  []byte
	RedirectURIs  []string `xorm:"redirect_uris"`
	GrantTypes    []string `xorm:"grant_types"`
	ResponseTypes []string `xorm:"response_types"`
	Public        bool

	User *user.User `xorm:"-"`

	CreatedAt *time.Time `xorm:"created"`
	UpdatedAt *time.Time `xorm:"updated"`
}

// TableName returns the table's name
func (client OAuth2Client) TableName() string {
	return "oauth2_clients"
}

// BeforeInsert will generate new UUID for the client id if not already set
func (client *OAuth2Client) BeforeInsert() {
	if len(client.ClientID) == 0 {
		id := uuid.NewV4()
		client.ClientID = id.String()
	}
}

// GetOAuth2ClientByClientID returns the client with the given client id
func GetOAuth2ClientByClientID(clientID string) (*OAuth2Client, error) {
	if len(clientID) == 0 {
		return nil, ErrOAuth2ClientDoesNotExist{ClientID: clientID}
	}
	client := &OAuth2Client{}
	if _, err := x.ID(clientID).Get(&client); err != nil {
		return nil, err
	}
	return client, nil
}
